import greenfoot.*;
import java.util.HashMap;
import java.util.Map;

/**
 * Handles Sounds. Usage:
 * 
 * Call from other classes
 *     Sound.play("explosion.wav")
 * and
 *     Sound.stop("explosion.wav");
 * or
 *     Sound.register("ACK", "ack.mp3");
 *     Sound.play("ACK");
 *     Sound.stop("ACK");
 * also
 *     Sound.stopAll();
 * also
 *     Sound.initializeDefaultSounds();
 *     Sound.play("SAD")
 */
public enum Sound
{
    ; // entry-less enum to prevent creation of any instances of static state class.

    static boolean soundIsOn = SzenarioKonfigurator.soundIsOn;
    private static final Map<String, GreenfootSound> sounds = new HashMap<>();

    public static void initializeDefaultSounds() {
        register("ACK", "peetie-ack.mp3");
        register("NO", "peetie-nein.wav");
        register("STEP1", "step1.wav");
        register("STEP2", "step2.wav");
        register("STEP3", "step3.wav");
        register("STEP4", "step4.wav");
        register("BATTERYEATEN", "batteryEaten.wav");
        register("YEAH!", "peetie-yeah.mp3");
        register("EXPLOSION", "explosion-2.wav");
        register("GRABEN", "graben-short.wav");
        register("ABSTURZ", "absturz.wav");
        register("STEINSCHLAG", "steinschlag-2.wav"); 
        register("RAKETENEHMEN", "raketenehmen.wav");
        register("ENDLOSSCHLEIFE", "endlosschleife-2.mp3");
        register("BING", "bing.wav");
        register("MISSILE", "rakete-2.wav");
        register("ABSCHUSS", "laser-blast-1.wav");
        register("PLOCK", "plock.wav");
        register("SHOOT", "photon-blast-1.wav");
        register("SMALLHIT", "kleine-explosion-2.wav");
        register("BGMUSIC", "EFM_MUSIC.mp3");
        register("LOADING", "loading.wav");
        register("GROWL", "growl.wav");        
        register("LAUNCHCOMPLETE", "LAUNCH.mp3");
        register("ENTSCHAERFEN", "entschaerfen.wav");
    }

    public static void register(final String fileName) {
        register(fileName, fileName);
    }

    public static void register(final String soundName, final String fileName) {
        final GreenfootSound alreadyRegisteredSound = sounds.get(soundName);
        if (alreadyRegisteredSound != null) {
            alreadyRegisteredSound.stop(); // make sure that old sound is stopped before replacing it.
        }
        sounds.put(soundName, create(fileName));
    }

    private static GreenfootSound create(final String fileName) {
        return new GreenfootSound(fileName);
    }

    public static void play(final String soundName) {
        final GreenfootSound sound = sounds.get(soundName);
        if(soundIsOn) {
            try {
                if (sound == null) { // register, if not yet existing.
                    register(soundName);
                }
                if (!sound.isPlaying()) {
                    sound.play();
                }
            } catch(Exception e) {
                System.out.println("Sound " + soundName + " wurde nicht gefunden");
            }
        }
    }

    public static void stop(final String soundName) {
        final GreenfootSound sound = get(soundName);
        if (sound != null) {
            sound.stop();
        }
    }

    public static void stopAll() {
        for(final GreenfootSound sound : sounds.values()) {
            sound.stop();
        }
    }

    private static GreenfootSound get(final String soundName) {
        final GreenfootSound result = sounds.get(soundName);
        if (result == null) {
            System.out.println("ERROR - Sound " + soundName + " was called but doesn't exist. Check class SoundHelper.");
        }
        return result;
    }

    public static void toggleSoundIsOn() {
        soundIsOn = !soundIsOn;
    }
}